@@ -1,18 +1,21 @@
+'''Declarative base class/factory and query property support.
+'''
 
-import functools
-from collections import namedtuple
-
-import sqlalchemy
 from sqlalchemy import inspect, orm, and_, or_
 from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
-import six
 
 from alchy import query, events
 from alchy.utils import classproperty, is_sequence, has_primary_key, camelcase_to_underscore
+from alchy._compat import iteritems
 
 
 class ModelMeta(DeclarativeMeta):
-    def __new__(cls, name, bases, dct):
+    '''ModelBase's metaclass which provides:
+
+        - Tablename autogeneration
+        - Declarative ORM events
+    '''
+    def __new__(mcs, name, bases, dct):
         # set __tablename__ (if not defined) to underscore version of class name
         if not dct.get('__tablename__') and not dct.get('__table__') is not None and has_primary_key(dct):
             dct['__tablename__'] = camelcase_to_underscore(name)
@@ -24,7 +27,7 @@ class ModelMeta(DeclarativeMeta):
         if not dct.get('__events__'):
             dct['__events__'] = {}
 
-        return DeclarativeMeta.__new__(cls, name, bases, dct)
+        return DeclarativeMeta.__new__(mcs, name, bases, dct)
 
     def __init__(cls, name, bases, dct):
         super(ModelMeta, cls).__init__(name, bases, dct)
@@ -62,43 +65,42 @@ class ModelBase(object):
         self.update(*args, **kargs)
 
     def __repr__(self):  # pragma: no cover
-        '''Default representation of model'''
         values = ', '.join(['{0}={1}'.format(c, repr(getattr(self, c))) for c in self.columns])
         return '<{0}({1})>'.format(self.__class__.__name__, values)
 
     def update(self, data_dict=None, strict=False, **kargs):
-        '''
-        Update model with arbitrary set of data
-        '''
+        '''Update model with arbitrary set of data.'''
 
         data = data_dict if isinstance(data_dict, dict) else kargs
 
         updatable_fields = self.strict_update_fields if strict else data.keys()
         relationships = self.relationships
 
-        for k, v in six.iteritems(data):
-            if hasattr(self, k) and k in updatable_fields:
+        for field, value in iteritems(data):
+            if hasattr(self, field) and field in updatable_fields:
                 # consider v a dict if any of its elements are a dict
-                v_is_dict = any([isinstance(_v, dict) for _v in v]) if is_sequence(v) else isinstance(v, dict)
-
-                attr = getattr(self, k)
-
-                if hasattr(attr, 'update') and v_is_dict:
-                    # nest calls to attr.update if available and input is a data dict
-                    attr.update(v)
+                if is_sequence(value):
+                    is_dict = any([isinstance(val, dict) for val in value])
                 else:
-                    if k in relationships and v_is_dict and not v:
+                    is_dict = isinstance(value, dict)
+
+                attr = getattr(self, field)
+
+                if hasattr(attr, 'update') and is_dict:
+                    # nest calls to attr.update if available and input is a data dict
+                    attr.update(value)
+                else:
+                    if field in relationships and is_dict and not value:
                         # typically, if v is {}, then we're usually updating a relationship attribute
                         # where the relationship has an empty/null value in the database
                         # (e.g. a frontend sends missing relationship attribute as {})
                         # but if we set a relationship attribute = {}, things blow up
                         # so instead, convert {} to None which is valid for standard relationship attribute
-                        v = None
-                    setattr(self, k, v)
+                        value = None
+                    setattr(self, field, value)
 
     def to_dict(self, refresh_on_empty=True):
-        '''
-        Return dict representation of model.
+        '''Return dict representation of model.
 
         Drill down to any relationships and serialize those too.
 
@@ -108,10 +110,10 @@ class ModelBase(object):
 
         However, with one exception: refresh if current __dict__ is empty.
         '''
-        d = {}
+        data = {}
         descriptors = self.descriptors
 
-        for field, value in six.iteritems(self.__dict__):
+        for field, value in iteritems(self.__dict__):
             if field not in descriptors:
                 # skip non-descriptors
                 continue
@@ -121,28 +123,26 @@ class ModelBase(object):
             elif is_sequence(value):
                 value = [v.to_dict() if hasattr(v, 'to_dict') else v for v in value]
 
-            d[field] = value
+            data[field] = value
 
-        if not d and refresh_on_empty and self.session:
+        if not data and refresh_on_empty and self.session:
             # Model's __dict__ is empty but has a session associated with it.
             # Most likely the model was previously committed which resets the __dict__ state.
             # Refreshing from database will repopulate the model's __dict__.
             self.refresh()
-            d = self.to_dict(refresh_on_empty=False)
+            data = self.to_dict(refresh_on_empty=False)
 
-        return d
+        return data
 
     def __iter__(self):
         '''Implement __iter__ so model can be converted to dict via dict(model)'''
-        return six.iteritems(self.to_dict())
+        return iteritems(self.to_dict())
 
     @property
     def strict_update_fields(self):
-        '''
-        Model fields which are allowed to be updated during strict mode
+        '''Model fields which are allowed to be updated during strict mode.
 
-        Default is to limit to table columns
-        Override as needed in child classes
+        Default is to limit to table columns. Override as needed in child class.
         '''
         return self.columns
 
@@ -152,15 +152,21 @@ class ModelBase(object):
 
     @classmethod
     def get_search(cls, search_dict, filter_fns):
+        '''Generic helper for applying a key-pairs from `search_dict` to key-pairs from `filter_fns`
+        where keys are equal.
+        '''
         filters = []
 
-        for key, filter_fn in [(k, v) for k, v in six.iteritems(filter_fns) if k in search_dict]:
+        for key, filter_fn in [(k, v) for k, v in iteritems(filter_fns) if k in search_dict]:
             filters.append(filter_fn(search_dict[key]))
 
         return filters
 
     @classmethod
     def advanced_search(cls, search_dict):
+        '''Return set of filters generated from passing each key-pair from `search_dict` to
+        the filter functions defined in `cls.__advanced_search__`.
+        '''
         filters = None
         if cls.__advanced_search__:
             _filters = cls.get_search(search_dict, cls.__advanced_search__)
@@ -172,6 +178,9 @@ class ModelBase(object):
 
     @classmethod
     def simple_search(cls, search_string):
+        '''Return set of filters generated from passing each space-delimited search term
+        from `search_string` to the filter functions defined in `cls.__simple_search__`.
+        '''
         filters = None
 
         if cls.__simple_search__:
@@ -208,21 +217,27 @@ class ModelBase(object):
         return orm.object_session(self)
 
     def flush(self, *args, **kargs):
+        '''Call `session.flush` on `self`'''
         self.session.flush([self], *args, **kargs)
 
     def save(self, *args, **kargs):
+        '''Call `session.add` on `self`'''
         self.session.add(self, *args, **kargs)
 
     def delete(self, *args, **kargs):
+        '''Call `session.delete` on `self`'''
         return self.session.delete(self, *args, **kargs)
 
     def expire(self, *args, **kargs):
+        '''Call `session.expire` on `self`'''
         self.session.expire(self, *args, **kargs)
 
     def refresh(self, *args, **kargs):
+        '''Call `session.refresh` on `self`'''
         self.session.refresh(self, *args, **kargs)
 
     def expunge(self, *args, **kargs):
+        '''Call `session.expunge` on `self`'''
         self.session.expunge(self, *args, **kargs)
 
     @classmethod
@@ -258,7 +273,7 @@ class ModelBase(object):
     @classproperty
     def descriptors(cls):
         '''Return all ORM descriptors'''
-        return [d for d in inspect(cls).all_orm_descriptors.keys() if not d.startswith('__')]
+        return [descr for descr in inspect(cls).all_orm_descriptors.keys() if not descr.startswith('__')]
 
     @classproperty
     def relationships(cls):
@@ -277,6 +292,9 @@ class ModelBase(object):
 
 
 class QueryProperty(object):
+    '''Query property accessor which gives a model access to query capabilities
+    via Model.query which is equivalent to session.query(Model)
+    '''
     def __init__(self, session):
         self.session = session
 
@@ -286,13 +304,16 @@ class QueryProperty(object):
             if not getattr(Model, 'query_class', None):
                 Model.query_class = query.Query
 
-            q = Model.query_class(mapper, session=self.session())
-            q.__model__ = Model
+            query_property = Model.query_class(mapper, session=self.session())
+            query_property.__model__ = Model
 
-            return q
+            return query_property
 
 
 def make_declarative_base(session=None, query_property=None, Model=None, Base=None):
+    '''Factory function for either creating a new declarative base class or extending
+    a previously defined one.
+    '''
     if Model is None:
         Base = Base or ModelBase
         Model = declarative_base(cls=Base, constructor=Base.__init__, metaclass=ModelMeta)
@@ -302,6 +323,10 @@ def make_declarative_base(session=None, query_property=None, Model=None, Base=No
 
 
 def extend_declarative_base(Model, session=None, query_property=None):
+    '''Extend a declarative base class with additional properties.
+
+    - Extend `Model` with query property accessor
+    '''
     # attach query attribute to Model if `session` object passed in
     if session:
         Model.query = query_property(session) if query_property else QueryProperty(session)
